//
//  DataManager.m
//  SciChartDemo
//
//  Created by Admin on 09.11.15.
//  Copyright © 2015 SciChart Ltd. All rights reserved.
//

#import "DataManager.h"
#import <SciChart/SciChart.h>

@implementation DataManager

+(void) loadXyData:(id<SCIXyDataSeries>)data From:(double)min To:(double)max Random:(DataManagerRandom)params {
    for (int j = min; j < max; j++) {
        double yValue = randf(j * params.minRandMul + params.minRandAdd, j * params.maxRandMul + params.maxRandAdd);
        yValue += j*params.addMul + params.add;
        if ( ([data xType] == SCIDataType_DateTime)
            && ([data yType] == SCIDataType_DateTime) )
        {
            [data appendX:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:j]) Y:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:yValue])];
        } else if ([data xType] == SCIDataType_DateTime) {
            [data appendX:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:j]) Y:SCIGeneric(yValue)];
        } else if ([data yType] == SCIDataType_DateTime) {
            [data appendX:SCIGeneric(j) Y:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:yValue])];
        } else {
            [data appendX:SCIGeneric(j) Y:SCIGeneric(yValue)];
        }
    }
}

+(void) loadXyData:(id<SCIXyDataSeries>)data From:(double)min To:(double)max Function:(XyDataFunction)func {
    for (int j = min; j < max; j++) {
        func(data,j);
    }
}

+(void) loadOhlcData:(id<SCIOhlcDataSeries>)data From:(double)min To:(double)max Function:(OhlcDataFunction)func {
    for (int j = min; j < max; j++) {
        func(data,j);
    }
}

+(void) loadPriceData:(id<SCIOhlcDataSeries>) dataSeries
             fileName:(NSString*) fileName
           isReversed:(BOOL) reversed
                count:(int) count{
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:fileName
                                                         ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile:filePath
                                               encoding:NSUTF8StringEncoding
                                                  error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    NSArray* subItems;
    if(!reversed)
        for (int i=0; i<count; i++) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            [dataSeries appendX:SCIGeneric(i)
                           Open:SCIGeneric([subItems[1] floatValue])
                           High:SCIGeneric([subItems[2] floatValue])
                            Low:SCIGeneric([subItems[3] floatValue])
                          Close:SCIGeneric([subItems[4] floatValue])];
        }
    else{
        int j=0;
        for (int i=count-1; i>=0; i--) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            [dataSeries appendX:SCIGeneric(j)
                           Open:SCIGeneric([subItems[1] floatValue])
                           High:SCIGeneric([subItems[2] floatValue])
                            Low:SCIGeneric([subItems[3] floatValue])
                          Close:SCIGeneric([subItems[4] floatValue])];
            ++j;
        }
    }
}

+(void) loadDataFromFile:(id<SCIXyDataSeries>) dataSeries
                fileName:(NSString*) fileName
              startIndex:(int) startIndex
               increment:(int) increment
                 reverse:(BOOL)reverse
{
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:fileName
                                                         ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile: filePath
                                               encoding:NSUTF8StringEncoding
                                                  error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    NSArray* subItems;
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"YYYY-MM-dd"];
    
    if (reverse) {
        for (int i = (int)(items.count - 1); i >= startIndex; i -= increment) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            NSDate * date =[dateFormatter dateFromString: subItems[0]];
            
            [dataSeries appendX:SCIGeneric(date)
                              Y: SCIGeneric([subItems[1] floatValue])];
        }
    } else {
        for (int i = startIndex; i < items.count; i += increment) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            NSDate * date =[dateFormatter dateFromString: subItems[0]];
            
            [dataSeries appendX:SCIGeneric(date)
                              Y: SCIGeneric([subItems[1] floatValue])];
        }
    }
}

+(void) loadDataFromFile:(id<SCIXyDataSeries>) dataSeries
                fileName:(NSString*) fileName{
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:fileName ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile: filePath encoding:NSUTF8StringEncoding error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    
    for (int i=0; i<items.count; i++) {
        [dataSeries appendX:SCIGeneric(i) Y: SCIGeneric([items[i] floatValue])];
    }
}

+ (void)putDefaultDataMultiPaneIntoDataSeries:(id<SCIXyDataSeries>)dataSeries dataCount:(NSInteger)dataCount {
    
    SCIGenericType yData;
    yData.floatData = arc4random_uniform(100);
    yData.type = SCIDataType_Float;
    int32_t prevoius = 1;
    for (int i = 0; i < dataCount; i++) {
        prevoius = randi(prevoius, prevoius+10);
        NSDate *date = [NSDate dateWithTimeIntervalSinceNow:60*60*24*prevoius];
        SCIGenericType xData = SCIGeneric(date);
        float value = yData.floatData + randf(-5.0, 5.0);
        yData.floatData = value;
        [dataSeries appendX:xData Y:yData];
    }
    
}

+ (NSArray<SCDMultiPaneItem *> *)loadPaneStockData {
    
    NSInteger count = 3000;
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:@"EURUSD_Daily"
                                                         ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile:filePath
                                               encoding:NSUTF8StringEncoding
                                                  error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    NSArray* subItems;
    
    NSMutableArray *array = [[NSMutableArray alloc] initWithCapacity:count];
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"YYYY-MM-dd"];
    
    //    if(!reversed)
    for (int i = 0; i<count; i++) {
        subItems = [items[i] componentsSeparatedByString:@","];
        
        SCDMultiPaneItem *item = [SCDMultiPaneItem new];
        item.dateTime = [dateFormatter dateFromString: subItems[0]];
        item.open = [subItems[1] doubleValue];
        item.high = [subItems[2] doubleValue];
        item.low = [subItems[3] doubleValue];
        item.close = [subItems[4] doubleValue];
        item.volume = [subItems[5] doubleValue];
        
        [array addObject:item];
        
    }
    
    return array;
    //    else{
    //        int j=0;
    //        for (int i=count-1; i>=0; i--) {
    //            subItems = [items[i] componentsSeparatedByString:@","];
    //
    //            [dataSeries appendX:SCIGeneric(j)
    //                           Open:SCIGeneric([subItems[1] floatValue])
    //                           High:SCIGeneric([subItems[2] floatValue])
    //                            Low:SCIGeneric([subItems[3] floatValue])
    //                          Close:SCIGeneric([subItems[4] floatValue])];
    //            ++j;
    //        }
    //    }
}

@end

@implementation SCDMultiPaneItem

@end

@implementation SCDMovingAverage {
    int _lenght;
    int _circIndex;
    bool _filled;
    double _current;
    double _oneOverLength;
    NSMutableArray <NSNumber *> *_circularBuffer;
    double _total;
}

- (instancetype)initWithLength:(int)length {
    self = [super init];
    if (self) {
        _lenght = length;
        _oneOverLength = 1.0/(double)length;
        _circularBuffer = [[NSMutableArray alloc] initWithCapacity:_lenght];
        _circIndex = -1;
        _current = NAN;
    }
    return self;
}

- (void)update:(double)value {
    
    double lostValue =  _circularBuffer[_circIndex].doubleValue;
    _circularBuffer[_circIndex] = @(value);
    
    // Maintain totals for Push function
    _total += value;
    _total -= lostValue;
    
    // If not yet filled, just return. Current value should be double.NaN
    if (!_filled)
    {
        _current = NAN;
        return;
    }
    
    // Compute the average
    double average = 0.0;
    for (int i = 0; i < _circularBuffer.count; i++)
    {
        average += _circularBuffer[i].doubleValue;
    }
    
    _current = average * _oneOverLength;
    
}

- (SCDMovingAverage*)push:(double)value {
    
    if (++_circIndex == _lenght) {
        _circIndex = 0;
    }
    
    
    double lostValue = _circIndex < _circularBuffer.count ? _circularBuffer[_circIndex].doubleValue : 0.0;
    _circularBuffer[_circIndex] = @(value);
    
    _total += value;
    _total -= lostValue;
    
    if (!_filled && _circIndex != _lenght-1) {
        _current = NAN;
        return self;
    }
    else {
        _filled = true;
    }
    
    _current = _total * _oneOverLength;
    
    return self;
    
}

@end

@implementation SCDMcadPointItem

@end


@implementation MarketDataService{
    NSDate *_startDate;
    int _timeFrameMinutes;
    int _tickTimerIntervals;
    RandomPriceDataSource *_generator;
}

-(instancetype) initWithStartDate:(NSDate *)startDate
                 TimeFrameMinutes:(int) timeFrameMinutes
               TickTimerIntervals:(int) tickTimerIntervals{
    self = [super init];
    if(self){
        _startDate = startDate;
        _timeFrameMinutes = timeFrameMinutes;
        _tickTimerIntervals = tickTimerIntervals;
        
        _generator = [[RandomPriceDataSource alloc]initWithCandleIntervalMinutes:_timeFrameMinutes SimulateDateGap:true TimeInterval:_tickTimerIntervals UpdatesPerPrice:25 RandomSeed:100 StartingPrice:30 StartDate:_startDate];
    }
    return self;
}

-(void) subscribePriceUpdate: (OnNewData) callback{
    _generator.updateData = callback;
    _generator.newData = callback;
    
    [_generator startGeneratePriceBars];
}

-(void) clearSubscriptions{
    if(_generator.IsRunning){
        [_generator stopGeneratePriceBars];
        [_generator clearEventHandlers];
    }
}

-(NSMutableArray *) getHistoricalData:(int) numberBars{
    NSMutableArray *prices = [[NSMutableArray alloc]initWithCapacity: numberBars];
    
    for (int i = 0; i < numberBars; i++){
        [prices addObject:[_generator getNextData]];
    }
    
    return prices;
}

-(SCDMultiPaneItem *) getNextBar{
    return [_generator tick];
}

@end

@implementation RandomPriceDataSource{
    NSTimer * _timer;
    double Frequency;
    int _candleIntervalMinutes;
    bool _simulateDateGap;
    
    SCDMultiPaneItem *_lastPriceBar;
    SCDMultiPaneItem *_initialPriceBar;
    double _currentTime;
    int _updatesPerPrice;
    int _currentUpdateCount;
    NSTimeInterval _openMarketTime;
    NSTimeInterval _closeMarketTime;
    
    int _randomSeed;
    double _timeInerval;
}

@synthesize IsRunning;

-(instancetype) initWithCandleIntervalMinutes:(int) candleIntervalMinutes
                              SimulateDateGap:(BOOL) simulateDateGap
                                 TimeInterval:(double) timeInterval
                              UpdatesPerPrice:(int) updatesPerPrice
                                   RandomSeed:(int) randomSeed
                                StartingPrice:(double) startingPrice
                                    StartDate:(NSDate *) startDate{
    self = [super init];
    if(self){
        Frequency = 1.1574074074074073E-05;
        _openMarketTime = 360;
        _closeMarketTime = 720;
        
        _candleIntervalMinutes = candleIntervalMinutes;
        _simulateDateGap = simulateDateGap;
        _updatesPerPrice = updatesPerPrice;
        
        _timeInerval = timeInterval;
        
        _initialPriceBar = [[SCDMultiPaneItem alloc]init];
        _initialPriceBar.close = startingPrice;
        _initialPriceBar.dateTime = startDate;
        
        _lastPriceBar = [[SCDMultiPaneItem alloc]init];
        _lastPriceBar.close = _initialPriceBar.close;
        _lastPriceBar.dateTime = _initialPriceBar.dateTime;
        _lastPriceBar.high = _initialPriceBar.close;
        _lastPriceBar.low = _initialPriceBar.close;
        _lastPriceBar.open = _initialPriceBar.close;
        _lastPriceBar.volume = 0;
        
        _randomSeed = randomSeed;
    }
    
    return self;
}

-(BOOL) IsRunning{
    return _timer.isValid;
}

-(void) startGeneratePriceBars{
    _timer = [NSTimer scheduledTimerWithTimeInterval: _timeInerval
                                              target:self
                                            selector:@selector(onTimerElapsed:)
                                            userInfo:nil
                                             repeats:YES];
}
-(void) stopGeneratePriceBars{
    if(_timer.isValid)
        [_timer invalidate];
}
-(SCDMultiPaneItem *) getNextData{
    
    return [self getNextRandomPriceBar];
}
-(SCDMultiPaneItem *) getUpdateData{
    double num = _lastPriceBar.close + ([self randf:0 max:_randomSeed] - 48)*(_lastPriceBar.close/1000.0);
    double high = num>_lastPriceBar.high ? num : _lastPriceBar.high;
    double low = num<_lastPriceBar.low ? num : _lastPriceBar.low;
    long volumeInc = ([self randf:0 max:_randomSeed]*3 + 2)*0.5;
    
    _lastPriceBar.high = high;
    _lastPriceBar.low = low;
    _lastPriceBar.close = num;
    _lastPriceBar.volume += volumeInc;
    
    return _lastPriceBar;
}

-(double) randf:(double) min max:(double) max {
    return ((double)arc4random() / ARC4RANDOM_MAX) * (max - min) + min;
}

-(SCDMultiPaneItem *) getNextRandomPriceBar{
    double close = _lastPriceBar.close;
    double num = ([self randf:0 max:_randomSeed] - 0.9) * _initialPriceBar.close / 30.0;
    double num2 = [self randf:0 max:_randomSeed];
    double num3 = _initialPriceBar.close + _initialPriceBar.close / 2.0 * sin(7.27220521664304E-06 * _currentTime) + _initialPriceBar.close / 16.0 * cos(7.27220521664304E-05 * _currentTime) + _initialPriceBar.close / 32.0 * sin(7.27220521664304E-05 * (10.0 + num2) * _currentTime) + _initialPriceBar.close / 64.0 * cos(7.27220521664304E-05 * (20.0 + num2) * _currentTime) + num;
    double num4 = fmax(close, num3);
    double num5 = arc4random_uniform(_randomSeed) * _initialPriceBar.close / 100.0;
    double high = num4 + num5;
    double num6 = fmin(close, num3);
    double num7 = arc4random_uniform(_randomSeed) * _initialPriceBar.close / 100.0;
    double low = num6 - num7;
    long volume = (long) (arc4random_uniform(_randomSeed)*30000 + 20000);
    NSDate *openTime = _simulateDateGap ? [self emulateDateGap:_lastPriceBar.dateTime] : _lastPriceBar.dateTime;
    NSDate *closeTime = [openTime addTimeInterval:_candleIntervalMinutes];
    
    SCDMultiPaneItem *candle = [[SCDMultiPaneItem alloc]init];
    candle.close = num3;
    candle.dateTime = closeTime;
    candle.high = high;
    candle.low = low;
    candle.volume = volume;
    candle.open = close;
    
    _lastPriceBar = [[SCDMultiPaneItem alloc]init];
    _lastPriceBar.close = candle.close;
    _lastPriceBar.dateTime = candle.dateTime;
    _lastPriceBar.high = candle.high;
    _lastPriceBar.low = candle.low;
    _lastPriceBar.open = candle.open;
    _lastPriceBar.volume = candle.volume;
    
    _currentTime += _candleIntervalMinutes ;
    return candle;
}

-(NSDate *) emulateDateGap: (NSDate *) candleOpenTime{
    NSDate *result = candleOpenTime;
    
    if ([candleOpenTime timeIntervalSince1970] > _closeMarketTime)
    {
        NSDate *dateTime = candleOpenTime;
        dateTime = [dateTime addTimeInterval: 500];
        result = [dateTime addTimeInterval:_openMarketTime];
    }
    while ([result timeIntervalSince1970] < 500)
    {
        result = [result addTimeInterval: 500];
    }
    return result;
}

-(void) onTimerElapsed{
    if (_currentUpdateCount < _updatesPerPrice)
    {
        _currentUpdateCount++;
        SCDMultiPaneItem *updatedData = [self getUpdateData];
        if (updatedData != nil)
        {
            _updateData(updatedData);
        }
    }
    else
    {
        _currentUpdateCount = 0;
        SCDMultiPaneItem *nextData = [self getNextData];
        if (nextData != nil)
        {
            _newData(nextData);
        }
    }
}

-(void) clearEventHandlers{
    
}

-(SCDMultiPaneItem *) tick{
    if (_currentUpdateCount < _updatesPerPrice)
    {
        _currentUpdateCount++;
        return [self getUpdateData];
    }
    else
    {
        _currentUpdateCount = 0;
        return [self getNextData];
    }
}


@end









