//
//  ColumnChartViewController.m
//  SciChartDemo
//
//  Created by Admin on 27.01.16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "ColumnChartView.h"
#import <SciChart/SciChart.h>
#import "DataManager.h"

@implementation ColumnChartView

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]init];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        
        [self initializeSurfaceData];
    }
    
    return self;
}

@synthesize sciChartSurfaceView;
@synthesize surface;
-(void) prepare {
    [surface free];
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
}

-(void) initializeSurfaceData {
    [self prepare];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:20];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFFFFFFF];
    
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.6];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"yAxis";
    [surface attachAxis:axis IsXAxis:NO];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    
    axis = [[SCIDateTimeAxis alloc] init];
    axis.axisId = @"xAxis";
    [((SCIDateTimeAxis*)axis) setTextFormatting:@"dd/MM/yyyy"];
    [axis setCursorTextFormatting:@"dd-MM-yyyy"];
    [axis setStyle: axisStyle];
    [surface attachAxis:axis IsXAxis:YES];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    
    SCIXAxisDragModifier * xDragModifier = [SCIXAxisDragModifier new];
    xDragModifier.axisId = @"xAxis";
    xDragModifier.dragMode = SCIAxisDragMode_Scale;
    xDragModifier.clipModeX = SCIZoomPanClipMode_None;
    
    SCIYAxisDragModifier * yDragModifier = [SCIYAxisDragModifier new];
    yDragModifier.axisId = @"yAxis";
    yDragModifier.dragMode = SCIAxisDragMode_Pan;
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    SCIZoomExtentsModifier * zem = [[SCIZoomExtentsModifier alloc] init];
    SCIRolloverModifier * rollover = [[SCIRolloverModifier alloc] init];
    
    [rollover setModifierName:@"Rollover Modifier"];
    [zem setModifierName:@"ZoomExtents Modifier"];
    [pzm setModifierName:@"PinchZoom Modifier"];
    [yDragModifier setModifierName:@"YAxis Drag Modifier"];
    [xDragModifier setModifierName:@"XAxis Drag Modifier"];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[xDragModifier, yDragModifier, pzm, zem, rollover]];
    surface.chartModifier = gm;
  
    id<SCIBrush2D> brush1 = [[SCIBrushLinearGradient alloc] initWithColorCodeStart:0xFF4fd3a9
                                                                        Finish:0xFF44b993
                                                                     Direction:SCILinearGradientDirection_Vertical];
    id<SCIPen2D> pen1 = [[SCIPenSolid alloc] initWithColorCode:0xFF232323 Width:0.4];
    id<SCIRenderableSeries> chart1 = [self getColumnRenderableSeries:brush1 borderPen:pen1 order:0];
    
    id<SCIBrush2D> brush2 = [[SCIBrushLinearGradient alloc] initWithColorCodeStart:0xFF3099fc
                                                                        Finish:0xFF287fd1
                                                                     Direction:SCILinearGradientDirection_Vertical];
    id<SCIPen2D> pen2 = [[SCIPenSolid alloc] initWithColorCode:0xFF232323 Width:0.4];
    id<SCIRenderableSeries> chart2 = [self getColumnRenderableSeries:brush2 borderPen:pen2 order:1];
    
    id<SCIBrush2D> brush3 = [[SCIBrushLinearGradient alloc] initWithColorCodeStart:0xFF3f3bd6
                                                                        Finish:0xFF3733bc
                                                                     Direction:SCILinearGradientDirection_Vertical];
    id<SCIPen2D> pen3 = [[SCIPenSolid alloc] initWithColorCode:0xFF232323 Width:0.4];
    id<SCIRenderableSeries> chart3 = [self getColumnRenderableSeries:brush3 borderPen:pen3 order:2];
    
    [surface attachRenderableSeries:chart1];
    [surface attachRenderableSeries:chart2];
    [surface attachRenderableSeries:chart3];
    
    [surface invalidateElement];
}

-(SCIFastColumnRenderableSeries *) getColumnRenderableSeries:(id<SCIBrush2D>) fillBrush
                                                borderPen: (id<SCIPen2D>) borderPen
                                                    order:(int) order{
    SCIXyDataSeries * columnDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_DateTime YType:SCIDataType_Float];
    
    //Getting Fourier dataSeries
    [DataManager loadDataFromFile:columnDataSeries
                         fileName:@"ColumnData"
                       startIndex:order
                        increment:3 reverse:NO];
    
    SCIFastColumnRenderableSeries * columnRenderableSeries = [[SCIFastColumnRenderableSeries alloc] init];
    
    
    
    columnDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    columnRenderableSeries.style.fillBrush = fillBrush;
    columnRenderableSeries.style.borderPen = borderPen;
    columnRenderableSeries.style.dataPointWidth = 0.3;
    
    columnRenderableSeries.xAxisId = @"xAxis";
    columnRenderableSeries.yAxisId = @"yAxis";
    
    [columnRenderableSeries setDataSeries:columnDataSeries];
    
    return columnRenderableSeries;
}

@end
