//
//  ECGChartView.m
//  SciChartDemo
//
//  Created by Admin on 21.03.16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "ECGChartView.h"
#import <SciChart/SciChart.h>
#include <math.h>
#include "DataManager.h"

#define ARC4RANDOM_MAX 0x100000000

@implementation ECGChartView{
    SCIXyDataSeries * dataSeries;
    SCIXyDataSeries * _sourceData;
    
    NSTimer *timer;
    double test10LastValue;
    int test10XCounter;
    BOOL _isBeat;
    BOOL _lastBeat;
    int _heartRate;
    NSDate *_lastBeatTime;
    int _currentIndex;
    int _totalIndex;
}

@synthesize sciChartSurfaceView;
@synthesize surface;

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]init];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        
        _currentIndex=0;
        _totalIndex =0;
        [self initializeSurfaceData];
    }
    
    return self;
}

-(void) prepare {
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
}

-(void) initializeSurfaceData {
    [self prepare];
    
    _sourceData = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Float YType:SCIDataType_Float];
    [DataManager loadDataFromFile:_sourceData fileName:@"WaveformData"];
    _lastBeatTime = [NSDate date];
    
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.6];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    float updateTime = 1.0f / 30.0f;
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    [axis setAutoRange:SCIAutoRange_Always];
    [axis setAnimatedChangeDuration: updateTime*2 ];
    [axis setAnimateVisibleRangeChanges: YES];
    [axis setAxisId: @"yAxis"];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:NO];
    
    axis = [[SCINumericAxis alloc] init];
    [axis setAutoRange:SCIAutoRange_Always];
    axis.axisId = @"xAxis";
    [axis setStyle: axisStyle];
    axis.animatedChangeDuration = updateTime*2;
    axis.animateVisibleRangeChanges = YES;
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:YES];
    
    SCIXAxisDragModifier * xDragModifier = [SCIXAxisDragModifier new];
    xDragModifier.axisId = @"xAxis";
    xDragModifier.dragMode = SCIAxisDragMode_Scale;
    xDragModifier.clipModeX = SCIZoomPanClipMode_None;
    
    SCIYAxisDragModifier * yDragModifier = [SCIYAxisDragModifier new];
    yDragModifier.axisId = @"yAxis";
    yDragModifier.dragMode = SCIAxisDragMode_Pan;
    
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    SCIZoomPanModifier * zpm = [[SCIZoomPanModifier alloc] init];
    SCIZoomExtentsModifier * zem = [[SCIZoomExtentsModifier alloc] init];
    
    [zpm setModifierName:@"PanZoom Modifier"];    
    [zem setModifierName:@"ZoomExtents Modifier"];
    [pzm setModifierName:@"PinchZoom Modifier"];
    [yDragModifier setModifierName:@"YAxis Drag Modifier"];
    [xDragModifier setModifierName:@"XAxis Drag Modifier"];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[xDragModifier, yDragModifier, pzm, zem, zpm]];
    surface.chartModifier = gm;
    
    [surface attachRenderableSeries: [self getECGRenderableSeries]];
    
    [surface invalidateElement];
}

- (void)willMoveToWindow:(UIWindow *)newWindow{
    [super willMoveToWindow: newWindow];
    if(timer == nil){
        timer = [NSTimer scheduledTimerWithTimeInterval:0.04
                                                 target:self
                                               selector:@selector(updateECGData:)
                                               userInfo:nil
                                                repeats:YES];}
    else{
        [timer invalidate];
        timer = nil;
    }
}

-(void)updateECGData:(NSTimer *)timer{
    for (int i = 0; i < 10; i++)
        [self AppendPoint:400];
    _isBeat = [[dataSeries yValues] valueAt:[dataSeries count]-3].doubleData > 0.5 ||
    [[dataSeries yValues] valueAt:[dataSeries count]-5].doubleData > 0.5||
    [[dataSeries yValues] valueAt:[dataSeries count]-8].doubleData > 0.5;
    
    if (_isBeat && !_lastBeat)
    {
        _heartRate = (int)(60.0 / [_lastBeatTime timeIntervalSinceNow]);
        _lastBeatTime = [NSDate date];
    }
    
}

-(void) AppendPoint:(double) sampleRate{
    if (_currentIndex >= [_sourceData count])
    {
        _currentIndex = 0;
    }
    
    // Get the next voltage and time, and append to the chart
    double voltage = [[_sourceData yValues] valueAt:_currentIndex].floatData*1000;
    double time = _totalIndex / sampleRate;
    [dataSeries appendX:SCIGeneric(time) Y:SCIGeneric(voltage)];
    [surface invalidateElement];
    _lastBeat = _isBeat;
    _currentIndex++;
    _totalIndex++;
}


-(SCIFastLineRenderableSeries*) getECGRenderableSeries{
    dataSeries = [[SCIXyDataSeries alloc] initFifoWithXType:SCIDataType_Float YType:SCIDataType_Float FifoSize:1500];
    
    dataSeries.seriesName = @"ECG";
    dataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    SCIFastLineRenderableSeries * ecgRenderableSeries = [[SCIFastLineRenderableSeries alloc] init];
    
    [ecgRenderableSeries.style setLinePen:[[SCIPenSolid alloc] initWithColorCode:0xFFf6e8b3 Width:0.5]];
    [ecgRenderableSeries setXAxisId: @"xAxis"];
    [ecgRenderableSeries setYAxisId: @"yAxis"];
    [ecgRenderableSeries setDataSeries:dataSeries];
    
    return ecgRenderableSeries;
}

@end
