//
//  HeatmapChartViewController.m
//  SciChartDemo
//
//  Created by Admin on 16.02.16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "HeatmapChartView.h"
#import <SciChart/SciChart.h>
#import "DataManager.h"

@implementation HeatmapChartView {
    SCIHeatMapDataSeries * heatmapDataSeries;
    int size;
    int increment;
    NSTimer *timer;
    float scale;
    BOOL _running;
}

@synthesize sciChartSurfaceView;
@synthesize surface;

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]init];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        scale = 0.1;
        [self initializeSurfaceData];
    }
    
    return self;
}

-(void) prepare {
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
}

-(void) initializeSurfaceData {
    [self prepare];
    
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.6];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Arial"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"yAxis";
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.05) Max:SCIGeneric(0.05)]];
    [surface attachAxis:axis IsXAxis:NO];
    
    axis = [[SCINumericAxis alloc] init];
    axis.axisId = @"xAxis";
    [axis setStyle: axisStyle];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.05) Max:SCIGeneric(0.05)]];
    [surface attachAxis:axis IsXAxis:YES];
    
    SCIXAxisDragModifier * xDragModifier = [SCIXAxisDragModifier new];
    xDragModifier.axisId = @"xAxis";
    xDragModifier.dragMode = SCIAxisDragMode_Scale;
    xDragModifier.clipModeX = SCIZoomPanClipMode_None;
    
    SCIYAxisDragModifier * yDragModifier = [SCIYAxisDragModifier new];
    yDragModifier.axisId = @"yAxis";
    yDragModifier.dragMode = SCIAxisDragMode_Pan;
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    SCIZoomExtentsModifier * zem = [[SCIZoomExtentsModifier alloc] init];
    SCITooltipModifier * tooltip = [[SCITooltipModifier alloc] init];
//    tooltip.style.tooltipSize = CGSizeMake(100, NAN);
    
    [tooltip setModifierName:@"ToolTip Modifier"];
    [zem setModifierName:@"ZoomExtents Modifier"];
    [pzm setModifierName:@"PinchZoom Modifier"];
    [yDragModifier setModifierName:@"YAxis Drag Modifier"];
    [xDragModifier setModifierName:@"XAxis Drag Modifier"];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[xDragModifier, yDragModifier, pzm, zem, tooltip]];
    surface.chartModifier = gm;
    
    [surface attachRenderableSeries:[self getHeatmapRenderableSeries]];
    
    [surface invalidateElement];
}

- (void)willMoveToWindow:(UIWindow *)newWindow{
    [super willMoveToWindow: newWindow];
    if(timer == nil) {
        timer = [NSTimer scheduledTimerWithTimeInterval:0.1
                                                 target:self
                                               selector:@selector(updateHeatmapData:)
                                               userInfo:nil
                                                repeats:YES];
    } else {
        [timer invalidate];
        timer = nil;
    }
}

-(void)updateHeatmapData:(NSTimer *)timer{
    float seriesPerPeriod = 30;
    float angle = (float)M_PI*scale/seriesPerPeriod;
    
    for (int x = 0; x < size; x++) {
        for (int y = 0; y < size; y++) {
            float v = (1 + sin(x * 0.04 + angle)) * 50 + (1 + sin(y * 0.1 + angle)) * 50 * (1 + sin(angle * 2));
            float r = sqrt(x * x + y * y);
            float exp = MAX(0, 1 - r * 0.008);
            float d= (v * exp + arc4random() % 2)/100;
            [[heatmapDataSeries data] setValue:SCIGeneric(d) AtX:x Y:y];
        }
    }
    scale += 0.5;
    [surface invalidateElement];
}

-(SCIHeatMapRenderableSeries*) getHeatmapRenderableSeries{
    increment = 1;
    SCIDoubleRange *range = [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0)
                                                     Max:SCIGeneric(1)];
    size = 100;
    
    heatmapDataSeries = [[SCIHeatMapDataSeries alloc] initWithTypeX:SCIDataType_Float
                                                                  Y:SCIDataType_Float
                                                                  Z:SCIDataType_Float
                                                              SizeX:size Y:size
                                                             RangeX:range Y:range];
    
    heatmapDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    for (int i=0; i<size; i++) {
        for (int j=0; j<size; j++) {
            [[heatmapDataSeries data]setValue:SCIGeneric((double)i*j/10) AtX:i Y:j];
        }
    }
    
    [heatmapDataSeries setSeriesName:@"Heatmap Series"];
    
    SCIHeatMapRenderableSeries * heatmapRenderableSeries = [[SCIHeatMapRenderableSeries alloc] init];
    [heatmapRenderableSeries.style setMax:SCIGeneric(1)];
    heatmapRenderableSeries.xAxisId = @"xAxis";
    heatmapRenderableSeries.yAxisId = @"yAxis";
    
    [heatmapRenderableSeries setDataSeries:heatmapDataSeries];
    return heatmapRenderableSeries;
}

@end
