//
//  NxMSeriesSpeedTestSciChart.m
//  ComparisonApp
//
//  Created by Yaroslav Pelyukh on 4/13/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "NxMSeriesSpeedTestSciChart.h"
#import <SciChart/SciChart.h>
#import "SpeedTest.h"
#import "RandomWalkGenerator.h"

#define ARC4RANDOM_MAX 0x100000000

@interface NxMSeriesSpeedTestSciChart () <SpeedTest>

@end

@implementation NxMSeriesSpeedTestSciChart{
    NSMutableArray * _series;
    SCINumericAxis * _xAxis;
    SCINumericAxis * _yAxis;
    NSMutableArray * _dataSeries;
    NSMutableArray * fpsData;
    NSTimer *timer;
    double deviation;
    double duration;
    double startTime;
    double timeUpdate;
    CFTimeInterval startTimeEvent;
    int frameCount;
    RandomWalkGenerator * randomWalkGenerator;
    int updateNumber;
    double rangeMin, rangeMax;
}

@synthesize delegate;
@synthesize chartProviderName;
@synthesize sciChartSurfaceView;
@synthesize surface;

-(NSArray*) getSeries {
    return _series;
}

-(NSArray*) getDataSeries {
    return _dataSeries;
}

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        fpsData = [[NSMutableArray alloc]init];
        self.sciChartSurfaceView = [[SCIChartSurfaceView alloc]init];
        
        [self.sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:self.sciChartSurfaceView];
        
        NSDictionary *layout = @{@"Charts":self.sciChartSurfaceView};
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[Charts]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[Charts]-(0)-|" options:0 metrics:0 views:layout]];
        
        _series = [NSMutableArray new];
        _dataSeries = [NSMutableArray new];
        
        self.chartProviderName = @"SciChart";
    }
    
    return self;
}


-(void) prepare {
    self.surface = [[SCIChartSurface alloc] initWithView: self.sciChartSurfaceView];
    
    [[self.surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[self.surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [self.surface.renderSurface setReduceCPUFrames:NO];
}

-(void) initializeSurfaceData:(TestParameters) testParameters {
    
    [self prepare];
    
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.5];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:TRUE];
    [axisStyle setDrawMajorBands:TRUE];
    
    _xAxis = [[SCINumericAxis alloc] init];
    [_xAxis setAxisId: @"xAxis"];
    [_xAxis setStyle: axisStyle];
//    [_xAxis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [_xAxis setAutoRange:SCIAutoRange_Once];
    [self.surface attachAxis:_xAxis IsXAxis:YES];
    
    _yAxis = [[SCINumericAxis alloc] init];
    [_yAxis setAxisId: @"yAxis"];
    [_yAxis setStyle: axisStyle];
//    [_yAxis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [_yAxis setAutoRange:SCIAutoRange_Once];
    [self.surface attachAxis:_yAxis IsXAxis:NO];
    self.surface.style.rightAxisAreaSize = 30;
    self.surface.style.leftAxisAreaSize = 5;
    self.surface.style.bottomAxisAreaSize = 30;
    self.surface.style.topAxisAreaSize = 5;
    self.surface.style.autoSizeAxes = NO;
    
    uint color = 0xFF4c8aff;

    //Getting Fourier dataSeries
    for(int series=0; series < testParameters.SeriesNumber; series++){
        
        SCIXyDataSeries * fourierDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Float YType:SCIDataType_Float];
        NSMutableArray* randomWalkData = [randomWalkGenerator GetRandomWalkSeries:testParameters.PointCount min:-0.5 max:0.5 includePrior:YES];
        
        for(int i=0; i<testParameters.PointCount; i++){
            double x = [[[randomWalkData objectAtIndex:0] objectAtIndex:i] doubleValue];
            double y = [[[randomWalkData objectAtIndex:1] objectAtIndex:i] doubleValue];
            [fourierDataSeries appendX:SCIGeneric(x) Y:SCIGeneric(y)];
        };
        
        color = (color + 0x100Ff0) | 0xFF000000;
        fourierDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
        
        SCIFastLineRenderableSeries * fourierRenderableSeries = [SCIFastLineRenderableSeries new];
        fourierRenderableSeries.style.linePen = [[SCIPenSolid alloc] initWithColorCode:color Width:0.5];
        fourierRenderableSeries.xAxisId = @"xAxis";
        fourierRenderableSeries.yAxisId = @"yAxis";
        [fourierRenderableSeries setDataSeries:fourierDataSeries];
        
        [self addSeries:fourierRenderableSeries];
        [self.surface invalidateElement];
    }
}

-(void) addSeries:(id<SCIRenderableSeries>)series {
    if (series.xAxisId == nil || series.yAxisId == nil) {
        series.xAxisId = _xAxis.axisId;
        series.yAxisId = _yAxis.axisId;
    }
    [_series addObject:series];
    [self.surface attachRenderableSeries:series];
}

#pragma SpeedTest implementation
-(void)runTest:(TestParameters)testParameters{
    rangeMin = NAN;
    rangeMax = NAN;
    randomWalkGenerator = [[RandomWalkGenerator alloc]init];
    [self initializeSurfaceData:testParameters];
    updateNumber = 0;
}

-(void)updateChart{
    [self.delegate chartExampleStarted];
    if(isnan(rangeMin)){
        rangeMin = SCIGenericDouble([[_yAxis visibleRange] min]);
        rangeMax = SCIGenericDouble([[_yAxis visibleRange] max]);
    }
    
    double scaleFactor = fabs(sin(updateNumber * 0.1)) + 0.5;
    [_yAxis setVisibleRange:[[SCIDoubleRange alloc] initWithMin:(SCIGeneric(rangeMin * scaleFactor))
                                                            Max:(SCIGeneric(rangeMax * scaleFactor))]];
    [self.surface invalidateElement];
    updateNumber++;
}


-(void)stopTest{
    [self.delegate processCompleted];
}

@end
