//
//  ScatterSpeedTestSciChart.m
//  ComparisonApp
//
//  Created by Yaroslav Pelyukh on 4/20/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "ScatterSpeedTestSciChart.h"
#import <SciChart/SciChart.h>
#import "SpeedTest.h"
#import "BrownianMotionGenerator.h"

#define ARC4RANDOM_MAX 0x100000000

@interface ScatterSpeedTestSciChart () <SpeedTest>

@end

@implementation ScatterSpeedTestSciChart{
    NSMutableArray * _series;
    SCINumericAxis * _xAxis;
    SCINumericAxis * _yAxis;
    NSMutableArray * _dataSeries;
    double deviation;
    SCIXyDataSeries * scatterDataSeries;
    BrownianMotionGenerator * randomWalkGenerator;
}

@synthesize delegate;
@synthesize chartProviderName;
@synthesize sciChartSurfaceView;
@synthesize surface;

-(NSArray*) getSeries {
    return _series;
}

-(NSArray*) getDataSeries {
    return _dataSeries;
}

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        randomWalkGenerator = [[BrownianMotionGenerator alloc]init];
        self.sciChartSurfaceView = [[SCIChartSurfaceView alloc]init];
        
        [self.sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:self.sciChartSurfaceView];
        
        NSDictionary *layout = @{@"SciChart":self.sciChartSurfaceView};
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        
        _series = [NSMutableArray new];
        _dataSeries = [NSMutableArray new];
        
        self.chartProviderName = @"SciChart";
    }
    
    return self;
}

-(void)willMoveToSuperview:(UIWindow *)newWindow{
    [super willMoveToWindow: newWindow];
}

-(double) randf:(double) min max:(double) max {
    return ((double)arc4random() / ARC4RANDOM_MAX) * (max - min) + min;
}

-(void) prepare {
    self.surface = [[SCIChartSurface alloc] initWithView: self.sciChartSurfaceView];
    
    [[self.surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[self.surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [self.surface.renderSurface setReduceCPUFrames:NO];
}

-(void) initializeSurfaceData:(TestParameters) testParameters {
    
    [self prepare];
    
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.5];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:TRUE];
    [axisStyle setDrawMajorBands:TRUE];
    
    _xAxis = [[SCINumericAxis alloc] init];
    [_xAxis setAxisId: @"xAxis"];
    [_xAxis setStyle: axisStyle];
//    [_xAxis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [_xAxis setAutoRange:SCIAutoRange_Once];
    [self.surface attachAxis:_xAxis IsXAxis:YES];
    
    _yAxis = [[SCINumericAxis alloc] init];
    [_yAxis setAxisId: @"yAxis"];
    [_yAxis setStyle: axisStyle];
//    [_yAxis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
//    [_yAxis setAutoRange:SCIAutoRange_Always];
    [_yAxis setVisibleRange:[[SCIDoubleRange alloc] initWithMin:(SCIGeneric(-50)) Max:(SCIGeneric(50))]];
    [self.surface attachAxis:_yAxis IsXAxis:NO];
    
    //Getting Fourier dataSeries
    scatterDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Double YType:SCIDataType_Double];
    
    //Getting dataSeries
    NSMutableArray* randomWalkData = [randomWalkGenerator getXyData:testParameters.PointCount :-50 :50];
    
    for(int i=0; i<testParameters.PointCount; i++){
        double x = [[[randomWalkData objectAtIndex:0] objectAtIndex:i] doubleValue];
        double y = [[[randomWalkData objectAtIndex:1] objectAtIndex:i] doubleValue];
        [scatterDataSeries appendX:SCIGeneric(x) Y:SCIGeneric(y)];
    };
    
    SCIXyScatterRenderableSeries * xyScatterRenderableSeries = [[SCIXyScatterRenderableSeries alloc] init];
    
    scatterDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    SCICoreGraphicsPointMarker * marker = [[SCICoreGraphicsPointMarker alloc] init];
    marker.width = 6;
    marker.height = 6;
    xyScatterRenderableSeries.style.pointMarker = marker;
    xyScatterRenderableSeries.xAxisId = _xAxis.axisId;
    xyScatterRenderableSeries.yAxisId = _yAxis.axisId;
    xyScatterRenderableSeries.dataSeries = scatterDataSeries;

    [self addSeries:xyScatterRenderableSeries];
    [self.surface invalidateElement];
}

-(void) addSeries:(id<SCIRenderableSeries>)series {
    if (series.xAxisId == nil || series.yAxisId == nil) {
        series.xAxisId = _xAxis.axisId;
        series.yAxisId = _yAxis.axisId;
    }
    [_series addObject:series];
    [self.surface attachRenderableSeries:series];
}

static double randf(double min, double max) {
    return ((double)arc4random() / ARC4RANDOM_MAX) * (max - min) + min;
}

#pragma SpeedTest implementation

-(void)runTest:(TestParameters)testParameters{
    [self initializeSurfaceData:testParameters];
}

-(void)updateChart{
    [self.delegate chartExampleStarted];
    
    for (int i=0; i<scatterDataSeries.count; i++){

        SCIGenericType x = [[scatterDataSeries xValues] valueAt:i];
        SCIGenericType y = [[scatterDataSeries yValues] valueAt:i];
        
        [scatterDataSeries updateAt:i
                             XValue:SCIGeneric(SCIGenericDouble(x) + randf(-1.0, 1.0))
                             YValue:SCIGeneric(SCIGenericDouble(y) + randf(-0.5, 0.5))];
    }
    
    [self.surface invalidateElement];
}


-(void)stopTest{
    [self.delegate processCompleted];
}

@end
