//
//  StackedMountainChartViewController.m
//  SciChartDemo
//
//  Created by Yaroslav Pelyukh on 2/28/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "StackedMountainChartView.h"
#import <SciChart/SciChart.h>
#import "DataManager.h"

@implementation StackedMountainChartView

@synthesize sciChartSurfaceView;
@synthesize surface;

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]init];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        
        [self initializeSurfaceData];
    }
    
    return self;
}

-(void) prepare {
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
}

-(void) initializeSurfaceData {
    [self prepare];
    
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.6];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"yAxis";
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:NO];
    
    axis = [[SCIDateTimeAxis alloc] init];
    axis.axisId = @"xAxis";
    [((SCIDateTimeAxis*)axis) setTextFormatting:@"dd/MM/yyyy"];
    [axis setStyle: axisStyle];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:YES];
    
    SCIXAxisDragModifier * xDragModifier = [SCIXAxisDragModifier new];
    xDragModifier.axisId = @"xAxis";
    xDragModifier.dragMode = SCIAxisDragMode_Scale;
    xDragModifier.clipModeX = SCIZoomPanClipMode_None;
    
    SCIYAxisDragModifier * yDragModifier = [SCIYAxisDragModifier new];
    yDragModifier.axisId = @"yAxis";
    yDragModifier.dragMode = SCIAxisDragMode_Pan;
    
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    SCIZoomExtentsModifier * zem = [[SCIZoomExtentsModifier alloc] init];
    SCIRolloverModifier * rollover = [[SCIRolloverModifier alloc] init];
    
    [rollover setModifierName:@"Rollover Modifier"];
    [zem setModifierName:@"ZoomExtents Modifier"];
    [pzm setModifierName:@"PinchZoom Modifier"];
    [yDragModifier setModifierName:@"Y Axis Drag Modifier"];
    [xDragModifier setModifierName:@"X Axis Drag Modifier"];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[xDragModifier, yDragModifier, pzm, zem, rollover]];
    surface.chartModifier = gm;
    
    [self attachStackedMountainRenderableSeries];
    
    [surface invalidateElement];
}

-(void) attachStackedMountainRenderableSeries {
    SCIXyDataSeries * mountainDataSeries1 = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_DateTime YType:SCIDataType_Float];
    
    mountainDataSeries1.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    [DataManager loadDataFromFile:mountainDataSeries1
                         fileName:@"FinanceData"
                       startIndex:1
                        increment:1 reverse:YES];
    
    SCIStackedMountainRenderableSeries * topMountainRenderableSeries = [[SCIStackedMountainRenderableSeries alloc] init];
    id<SCIBrush2D> brush1 = [[SCIBrushLinearGradient alloc] initWithColorCodeStart:0x88ffffff
                                                                       Finish:0xDDDBE0E1
                                                                    Direction:SCILinearGradientDirection_Vertical];
    id<SCIPen2D> pen1 = [[SCIPenSolid alloc] initWithColorCode:0xFFffffff Width:0.5];
    [topMountainRenderableSeries.style setAreaBrush: brush1];
    [topMountainRenderableSeries.style setBorderPen: pen1];
    [topMountainRenderableSeries setXAxisId: @"xAxis"];
    [topMountainRenderableSeries setYAxisId: @"yAxis"];
    [topMountainRenderableSeries setDataSeries:mountainDataSeries1];

    
    SCIXyDataSeries * mountainDataSeries2 = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_DateTime YType:SCIDataType_Float];
    
    mountainDataSeries2.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    [DataManager loadDataFromFile:mountainDataSeries2
                         fileName:@"FinanceData"
                       startIndex:1
                        increment:1 reverse:YES];
    
    id<SCIBrush2D> brush2 = [[SCIBrushLinearGradient alloc] initWithColorCodeStart:0x88af9A90
                                                                        Finish:0x88af9A43
                                                                     Direction:SCILinearGradientDirection_Vertical];
    id<SCIPen2D> pen2 = [[SCIPenSolid alloc] initWithColorCode:0xFFffffff Width:0.5];
    SCIStackedMountainRenderableSeries * bottomMountainRenderableSeries = [[SCIStackedMountainRenderableSeries alloc] init];
    [bottomMountainRenderableSeries.style setAreaBrush: brush2];
    [bottomMountainRenderableSeries.style setBorderPen: pen2];
    [bottomMountainRenderableSeries setXAxisId: @"xAxis"];
    [bottomMountainRenderableSeries setYAxisId: @"yAxis"];
    [bottomMountainRenderableSeries setDataSeries:mountainDataSeries2];
    
    SCIStackedGroupSeries *stackedGroup = [[SCIStackedGroupSeries alloc] init];
    [stackedGroup setXAxisId: @"xAxis"];
    [stackedGroup setYAxisId: @"yAxis"];
    [stackedGroup addSeries:bottomMountainRenderableSeries];
    [stackedGroup addSeries:topMountainRenderableSeries];
    [surface attachRenderableSeries:stackedGroup];
}

@end
