//
//  UsingRolloverModifierChartView.m
//  SciChartDemo
//
//  Created by Yaroslav Pelyukh on 7/23/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "UsingRolloverModifierChartView.h"
#import "InterpolationTurnOnOff.h"

@implementation UsingRolloverModifierChartView

@synthesize sciChartSurfaceView;
@synthesize surface;

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]initWithFrame:frame];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        
        __weak typeof(self) wSelf = self;
        
        InterpolationTurnOnOff * panel = (InterpolationTurnOnOff*)[[[NSBundle mainBundle] loadNibNamed:@"InterpolationTurnOnOff" owner:self options:nil] firstObject];
        
        panel.onUseInterpolationClicked = ^() { [wSelf turnOnOffInterpolation]; };
        
        [self addSubview:panel];
        [self addSubview:sciChartSurfaceView];
        
        panel.translatesAutoresizingMaskIntoConstraints = NO;
        
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView, @"Panel":panel};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[Panel(63)]-(0)-[SciChart]-(0)-|"
                                                                     options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|"
                                                                     options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[Panel]-(0)-|"
                                                                     options:0 metrics:0 views:layout]];
        
        [self initializeSurfaceData];
    }
    
    return self;
}

-(void) turnOnOffInterpolation{
    for (SCIFastLineRenderableSeries *series in [surface renderableSeries]) {
        [[series hitTestProvider] setHitTestMode:[[series hitTestProvider] hitTestMode] == SCIHitTest_Vertical ? SCIHitTest_VerticalInterpolate : SCIHitTest_Vertical];
    }
}

-(void) prepare {
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
}

-(void) initializeSurfaceData {
    [self prepare];
    [self addAxes];
    [self addRolloverModifierModifiers];
    [self initializeSurfaceRenderableSeries];
}

-(void) addAxes{
    SCIPenSolid * majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.5];
    SCIBrushSolid * gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid * minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"yAxis";
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:NO];
    
    axis = [[SCINumericAxis alloc] init];
    axis.axisId = @"xAxis";
    [axis setStyle: axisStyle];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:YES];
}

-(void) addRolloverModifierModifiers{
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    [pzm setModifierName:@"PinchZoom Modifier"];
    
    SCIZoomExtentsModifier * zem = [[SCIZoomExtentsModifier alloc] init];
    [zem setModifierName:@"ZoomExtents Modifier"];
  
    SCIRolloverModifier * rollover = [[SCIRolloverModifier alloc] init];
    rollover.style.tooltipSize = CGSizeMake(200, NAN);
    [rollover setModifierName:@"Rollover Modifier"];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[ pzm, zem, rollover]];
    surface.chartModifier = gm;
}

-(void) initializeSurfaceRenderableSeries{
   
    SCIEllipsePointMarker * ellipsePointMarker = [[SCIEllipsePointMarker alloc]init];
    [ellipsePointMarker setDrawBorder:YES];
    [ellipsePointMarker setFillBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFFd6ffd7]];
    [ellipsePointMarker setHeight:5];
    [ellipsePointMarker setWidth:5];
    
    [surface attachRenderableSeries: [self getFastLineRenderableSeries: ellipsePointMarker :1.0 :0xFFd7b9a1]];
    [surface attachRenderableSeries: [self getFastLineRenderableSeries: ellipsePointMarker :0.5 :0xFF00540b]];
    [surface attachRenderableSeries: [self getFastLineRenderableSeries: nil :0 :0xFFa66e38]];
    
    [surface invalidateElement];
}

-(SCIFastLineRenderableSeries *) getFastLineRenderableSeries: (SCIEllipsePointMarker *) pointMarker :(double) amplitude :(uint) colorCode{
    
    SCIXyDataSeries * fourierDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Float YType:SCIDataType_Float];
    
    double count = 100.0;
    double k = 2 * M_PI / 30.0;
    for (int i = 0; i < (int) count; i++) {
        double phi = k * i;
        [fourierDataSeries appendX:SCIGeneric(i) Y:SCIGeneric((amplitude + i / count) * sin(phi))];
    }
    
    fourierDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    SCIFastLineRenderableSeries * fourierRenderableSeries = [SCIFastLineRenderableSeries new];
    fourierRenderableSeries.style.linePen = [[SCIPenSolid alloc] initWithColorCode:colorCode Width:1.0];
    fourierRenderableSeries.xAxisId = @"xAxis";
    fourierRenderableSeries.yAxisId = @"yAxis";
    [fourierRenderableSeries setDataSeries:fourierDataSeries];
    
    [[fourierRenderableSeries hitTestProvider] setHitTestMode: SCIHitTest_VerticalInterpolate];
    
    if(pointMarker){
        [fourierRenderableSeries.style setDrawPointMarkers:true];
        [fourierRenderableSeries.style setPointMarker:pointMarker];
    }
    
    return fourierRenderableSeries;
}

@end
