//
//  SCSDataManager.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/2/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSDataManager {
    
    
    
    static func loadData<DataSeriesType: SCIXyDataSeriesProtocol>(intoDataSeries dataSeries: DataSeriesType,
                         fileName: String,
                         startIndex: Int,
                         increment: Int,
                         reverse: Bool) {
        
        
        if let resourcePath = NSBundle.mainBundle().resourcePath {
            let filePath = resourcePath+"/"+fileName+".txt"
            do {
                let contentFile = try NSString(contentsOfFile: filePath, usedEncoding: nil) as String
                
                let items = contentFile.componentsSeparatedByString("\n")
                
                let dateFormatter = NSDateFormatter()
                dateFormatter.dateFormat = "YYYY-MM-dd"
                
                
                
                if reverse {
                    var i = items.count-1
                    while i >= startIndex {
                        
                        let subItems = items[i].componentsSeparatedByString(",")
                        
                        var date = dateFormatter.dateFromString(subItems[0])
                        var value = Float(subItems[1])
                        
                        dataSeries.appendX(SCI_constructGenericTypeWithInfo(&date, .DateTime),
                                           y: SCI_constructGenericTypeWithInfo(&value, .Float))
                        
                        i = i - increment
                    }
                    
                }
                else {
                   var i = startIndex
                    while i < items.count  {
                        
                        let subItems = items[i].componentsSeparatedByString(",")
                        
                        var date = dateFormatter.dateFromString(subItems[0])
                        var value = Float(subItems[1])
                        
                        dataSeries.appendX(SCI_constructGenericTypeWithInfo(&date, .DateTime),
                                           y: SCI_constructGenericTypeWithInfo(&value, .Float))
                        
                        i = i + 1
                    }
                }
            }
            catch {
                
                
            }
        }
    }
    
    static func putDataInto(dataSeries: SCIXyDataSeries) {
        let dataCount = 20
        var i = 0
        while i <= dataCount {
            let x = 10.0*Float(i)/Float(dataCount)
            let y = arc4random_uniform(UInt32(dataCount))
            var xValue = Float(x)
            var yValue = Float(y)
            dataSeries.appendX(SCI_constructGenericTypeWithInfo(&xValue, .Float),
                               y: SCI_constructGenericTypeWithInfo(&yValue, .Float))
            i = i + 1
        }
    }
    
    static func putFourierDataInto(dataSeries: SCIXyDataSeries) {
        let dataCount = 1000
        var i = 0
        while i <= dataCount {
            var x = Float(10.0*Float(i)/Float(dataCount))
            var y = 2 * sin(x)+10
            dataSeries.appendX(SCI_constructGenericTypeWithInfo(&x, .Float),
                               y: SCI_constructGenericTypeWithInfo(&y, .Float))
            i = i + 1
        }
    }
    
}
