//
//  SCSAnnotationsView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/6/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSAnnotationsChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addDefaultModifiers()
        addAnnotations()
    }
    
    // MARK: Private Functions
    
    private func addAxis() {
        
        let axisStyle = generateDefaultAxisStyle()
        
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
        
    }
    
    private func addAnnotations() {
        
        let box = SCIBoxAnnotation()
        box.xAxisId = axisXId
        box.yAxisId = axisYId
        box.coordMode = .Relative
        box.xStart = SCIGeneric(0.25)
        box.yStart = SCIGeneric(0.25)
        box.xEnd = SCIGeneric(0.5)
        box.yEnd = SCIGeneric(0.5)
        box.style.fillBrush = SCIBrushSolid(colorCode: 0x30FF7000)
        
        let lineAnnotation = SCILineAnnotation()
        lineAnnotation.xAxisId = axisXId
        lineAnnotation.yAxisId = axisYId
        lineAnnotation.coordMode = .Relative
        lineAnnotation.xStart = SCIGeneric(0.1)
        lineAnnotation.yStart = SCIGeneric(0.1)
        lineAnnotation.xEnd = SCIGeneric(0.2)
        lineAnnotation.yEnd = SCIGeneric(0.1)
        lineAnnotation.style.linePen = SCIPenSolid(colorCode: 0xFF4c8aff, width: 2)
        
        let textAnnotation = SCITextAnnotation()
        textAnnotation.xAxisId = axisXId
        textAnnotation.yAxisId = axisYId
        textAnnotation.coordMode = .Relative
        textAnnotation.xPosition = SCIGeneric(0.1)
        textAnnotation.yPosition = SCIGeneric(0.5)
        textAnnotation.text = "An amazing TextAnnotation"
        textAnnotation.style.textStyle = chartSurface.xAxes.getAxisById(axisXId).style.labelStyle
        
        let annotationGroup = SCIAnnotationGroup(childAnnotations: [box, lineAnnotation, textAnnotation])
        
        chartSurface.annotation = annotationGroup
        chartSurface.invalidateElement()
        
        
    }
    
    
}
