//
//  SCSHeatmapChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/3/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSHeatmapChartView: SCSBaseChartView {
    
    var heatmapDataSeries : SCIHeatMapDataSeries!
    var size: Int32 = 100
    var increment = 1
    var timer: NSTimer!
    var scale = 0.1

    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addModifiers()
        addDataSeries()
    }
    
    override func willMoveToSuperview(newSuperview: UIView?) {
        super.willMoveToSuperview(newSuperview)
        if timer == nil {
            timer = NSTimer.scheduledTimerWithTimeInterval(0.1,
                                                           target: self,
                                                           selector: "updateHeatmapData",
                                                           userInfo: nil,
                                                           repeats: true)
        }
    }
    
    func addModifiers() {
        
        let xAxisDragmodifier = SCIXAxisDragModifier()
        xAxisDragmodifier.modifierName = xAxisDragModifierName
        xAxisDragmodifier.axisId = axisXId
        xAxisDragmodifier.dragMode = .Scale
        xAxisDragmodifier.clipModeX = .None
        
        let yAxisDragmodifier = SCIYAxisDragModifier()
        yAxisDragmodifier.modifierName = yAxisDragModifierName
        yAxisDragmodifier.axisId = axisYId
        yAxisDragmodifier.dragMode = .Pan
        
        let extendZoomModifier = SCIZoomExtentsModifier()
        extendZoomModifier.modifierName = extendZoomModifierName
        
        let pinchZoomModifier = SCIPinchZoomModifier()
        pinchZoomModifier.modifierName = pinchZoomModifierName
        
        let tooltipModifier = SCITooltipModifier()
        tooltipModifier.modifierName = "TooltipModifier"
        tooltipModifier.style.colorMode = .SeriesColor
        //        rolloverModifier.style.tooltipSize = CGSizeMake(200, CGFloat.NaN)
        
        let groupModifier = SCIModifierGroup(childModifiers: [xAxisDragmodifier, yAxisDragmodifier, pinchZoomModifier, extendZoomModifier, tooltipModifier])
        
        chartSurface.chartModifier = groupModifier
    }
    
    override func removeFromSuperview() {
        super.removeFromSuperview()
        if timer != nil {
            timer.invalidate()
        }
    }
    
    // MARK: Private Functions
    
    private func addAxis() {
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
    }
    
    private func addDataSeries() {
        
        let range = SCIDoubleRange(min: SCIGeneric(0), max: SCIGeneric(1))
        heatmapDataSeries = SCIHeatMapDataSeries(typeX: .Float,
                                                 y: .Float,
                                                 z: .Float,
                                                 sizeX: size,
                                                 y: size,
                                                 rangeX: range,
                                                 y: range)
        heatmapDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        var i: Int32 = 0
        while i < size {
            var j: Int32 = 0
            while j < size {
                heatmapDataSeries.data().setValue(SCIGeneric(Float(i*j/10)), atX: i, y: j)
                j += 1
            }
            i += 1
        }
        
        heatmapDataSeries.seriesName = "HeatSeriesName"
        
        let heatRenderableSeries = SCIHeatMapRenderableSeries()
        heatRenderableSeries.style.max = SCIGeneric(1)
        heatRenderableSeries.xAxisId = axisXId
        heatRenderableSeries.yAxisId = axisYId
        heatRenderableSeries.dataSeries = heatmapDataSeries
        
        chartSurface.attachRenderableSeries(heatRenderableSeries)
        chartSurface.invalidateElement()
        
    }
    
    func updateHeatmapData() {
        
        let seriesPeriod = 30.0
        let angle = M_PI*scale/seriesPeriod
        
        var i: Int32 = 0
        while i < size {
            var j: Int32 = 0
            while j < size {
                let x = Double(i)
                let y = Double(j)
                var v = (1.0 + sin(x * 0.04 + angle)) * 50.0
                v = v +  (1.0 + sin(y * 0.1 + angle)) * 50.0 * (1.0 + sin(angle * 2.0))
                let r = sqrt(x * x + y * y)
                var exp = (1.0 - r * 0.008)
                exp = exp > 0.0 ? exp : 0.0
                let d = (v * exp + Double(arc4random() % 2))/100.0
                heatmapDataSeries.data().setValue(SCIGeneric(d), atX:i, y: j)
                j += 1
            }
            i += 1
        }
        scale += 0.5
        chartSurface.invalidateElement()
        
    }
    

    
    
}
