//
//  SCSLinePerformanceChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/6/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSLinePerformanceChartView: UIView {
    
    let sciChartView = SCSBaseChartView()
    var controlPanel : SCSLinePerformanceControlPanelView!
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        completeConfiguration()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    // MARK: Overrided Functions
    
    override func awakeFromNib() {
        super.awakeFromNib()
        completeConfiguration()
    }

    // MARK: Private Functions
    
    func completeConfiguration() {
        configureChartSuraface()
        addAxis()
        sciChartView.addDefaultModifiers()
    }
    
    private func addPanel() {
        let panel = NSBundle.mainBundle().loadNibNamed("SCSLinePerformanceControlPanelView",
                                                       owner: self,
                                                       options: nil).first
        
        if let panelValid = panel as? SCSLinePerformanceControlPanelView {
            
            controlPanel = panelValid
            
            controlPanel.translatesAutoresizingMaskIntoConstraints = false
            
            controlPanel.clearButton.addTarget(self,
                                             action: "clearAction",
                                             forControlEvents: .TouchUpInside)
            
            controlPanel.add100kButton.addTarget(self,
                                               action: "add100k",
                                               forControlEvents: .TouchUpInside)
            
            controlPanel.add1mButton.addTarget(self,
                                             action: "add1m",
                                             forControlEvents: .TouchUpInside)
            addSubview(controlPanel)
            
        }
    }
    
    private func configureChartSuraface() {
        
        sciChartView.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView)
        
        addPanel()
        
        let layoutDictionary = ["SciChart" : sciChartView, "Panel" : controlPanel]
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|-(0)-[Panel(43)]-(0)-[SciChart]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[Panel]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
    }
    
    private func addAxis() {
        let style = sciChartView.generateDefaultAxisStyle()
        sciChartView.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: style, axisId: sciChartView.axisXId), isXAxis: true)
        sciChartView.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: style, axisId: sciChartView.axisYId), isXAxis: false)
    }
    
    private func addSeriesWith(count: Int32, colorCode: UInt32) {
        let dataSeries = SCIXyDataSeries(XType: .Float, YType: .Double)
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        
        let y = Double(arc4random_uniform(100))
        
        var i: Int32 = 0
        while i < count {
            let x = SCIGeneric(i)
            let yValue = y + randomize(-5.0, max: 5.0)
            dataSeries.appendX(x, y: SCIGeneric(yValue))
            i += 1
        }
        
        let renderableSeries = SCIFastLineRenderableSeries()
        renderableSeries.style.linePen = SCIPenSolid(colorCode: colorCode, width: 0.5)
        renderableSeries.dataSeries = dataSeries
        renderableSeries.xAxisId = sciChartView.axisXId
        renderableSeries.yAxisId = sciChartView.axisYId
        
        sciChartView.chartSurface.attachRenderableSeries(renderableSeries)
        sciChartView.chartSurface.invalidateElement()
//        sciChartView.chartSurface.zoomExtents()
    }
    
    private func randomize(min: Double, max: Double) -> Double {
        return Double(arc4random() / 10000000) * (max - min) + min
    }
    

    // MARK: Button Actions
    
    func clearAction() {
        let series = sciChartView.chartSurface.renderableSeries
        for item in series {
            if let renderSeries = item as? SCIRenderableSeries {
                sciChartView.chartSurface.detachRenderableSeries(renderSeries)
            }
        }
  
    }
    
    func add100k() {
        addSeriesWith(100000+1, colorCode: 0xFF4fd3a9)
    }
    
    func add1m() {
        addSeriesWith(1000000+1, colorCode: 0xFF3099fc)
    }
    
}
