//
//  SCSNxMSeriesSpeedTestSciChart.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/29/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class SCSNxMSeriesSpeedTestSciChart: SCSBaseChartView, SpeedTest {
    
    let randomWalkGenerator = RandomWalkGenerator()
    
    var delegate: DrawingProtocolDelegate!
    var chartProviderName: String! = "SciChart"
    var parameters: TestParameters!
    var rangeMin = Double.NaN
    var rangeMax = Double.NaN
    var updateNumber = 0
    
    // MARK: SpeedTestProtocol
    
    func runTest(testParameters: TestParameters) {
        parameters = testParameters
        addSeries()
    }
    
    func updateChart() {
        delegate.chartExampleStarted!()
        
        if rangeMin.isNaN {
            rangeMin = SCIGenericDouble(chartSurface.yAxis.visibleRange.min)
            rangeMax = SCIGenericDouble(chartSurface.yAxis.visibleRange.max)
        }
        
        let scaleFactor = fabs(sin(Double(updateNumber)*0.1)) + 0.5
        
        chartSurface.yAxis.visibleRange = SCIDoubleRange(min: SCIGeneric(rangeMin * scaleFactor),
                                                         max: SCIGeneric(rangeMax * scaleFactor))
        
        chartSurface.invalidateElement()
        
        updateNumber += 1
    }
    
    func stopTest() {
        delegate.processCompleted!()
    }
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()

    }
    
    // MARK: Private Functions
    
    private func addAxes() {
        
        let axisStyle = generateDefaultAxisStyle()
        
        let axisX = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId);
        axisX.autoRange = .Once
        axisX.animatedChangeDuration = 1.0/30.0*2
        axisX.animateVisibleRangeChanges = true
        chartSurface.xAxis = axisX
        chartSurface.attachAxis(axisX, isXAxis: true)
        
        let axisY = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId);
        axisY.autoRange = .Once
        axisY.animatedChangeDuration = 1.0/30.0*2
        axisY.animateVisibleRangeChanges = true
        chartSurface.yAxis = axisY
        chartSurface.attachAxis(axisY, isXAxis: false)
        
        addDefaultModifiers()
        
    }
    
    private func addSeries() {
        var color: UInt32 = 0xFF4c8aff
        var series: Int32 = 0
        while series < parameters.SeriesNumber {
            
            let dataSeries = SCIXyDataSeries(XType: .Double, YType: .Double)
            dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
            let randomData: [NSArray] = NSArray.init(array: randomWalkGenerator.GetRandomWalkSeries(parameters.PointCount, min: -0.5, max: 0.5, includePrior: true)) as! [NSArray]
            
            var i: Int = 0
            while i < Int(parameters.PointCount) {
                
                let x = randomData[0][i].doubleValue
                let y = randomData[1][i].doubleValue
                
                dataSeries.appendX(SCIGeneric(x), y: SCIGeneric(y))
                
                i += 1
            }
            
            
            let renderSeries = SCIFastLineRenderableSeries()
            renderSeries.dataSeries = dataSeries
            renderSeries.xAxisId = axisXId
            renderSeries.yAxisId = axisYId
            renderSeries.style.linePen = SCIPenSolid(colorCode: color, width: 0.5)
            
            chartSurface.attachRenderableSeries(renderSeries)
            
            color = (color + 0x100F2) | 0xFF000000;
            
            series += 1
            
        }
        
        chartSurface.invalidateElement()
        
    }
    
}

