//
//  SCSRealtimeTickingStockChartView.swift
//  SciChartSwiftDemo
//
//  Created by Yaroslav Pelyukh on 7/19/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSRealtimeTickingStockChartView: UIView {
    
    let sciChartView1 = SCSBaseChartView()
    let sciChartView2 = SCSBaseChartView()
    
    var counter:Int32 = 0;
    let seriesCount:Int32 = 40;
    var countUpdater:Int32;
    
    let ohlcDataSeries : SCIOhlcDataSeries
    let mountainDataSeries : SCIXyDataSeries
    var timer:NSTimer!;
    
    let _lastMarker = SCIAxisMarkerAnnotation();
    let _lastUpColor = UIColor.fromColorCode(0xFF4440a6);
    let _lastDownColor = UIColor.fromColorCode(0xFF3ba13d);
    let _averageMarker = SCIAxisMarkerAnnotation();
    
    
    override init(frame: CGRect) {
        ohlcDataSeries = SCIOhlcDataSeries(fifoWithXType: .Float, YType: .Float, fifoSize: seriesCount);
        mountainDataSeries = SCIXyDataSeries(fifoWithXType: .Float, YType: .Float, fifoSize: seriesCount);
        countUpdater = seriesCount;
        super.init(frame: frame)
        completeConfiguration()
    }
    
    required init?(coder aDecoder: NSCoder) {
        ohlcDataSeries = SCIOhlcDataSeries(fifoWithXType: .Float, YType: .Float, fifoSize: seriesCount);
        mountainDataSeries = SCIXyDataSeries(fifoWithXType: .Float, YType: .Float, fifoSize: seriesCount);
        countUpdater = seriesCount;
        super.init(coder: aDecoder)
    }
    
    override func willMoveToWindow(newWindow: UIWindow?) {
        super.willMoveToWindow(newWindow);
        
        if(timer == nil){
            timer = NSTimer.scheduledTimerWithTimeInterval(0.7,
                                                           target: self,
                                                           selector: "updateData",
                                                           userInfo: nil,
                                                           repeats: true)}
    }
    
    override func removeFromSuperview() {
        super.removeFromSuperview()
        if timer != nil {
            timer.invalidate()
        }
    }
    
    func updateData(){
        
        //        if(ohlcDataSeries.xValues().count() != 0)
        
        if(counter == seriesCount){
            countUpdater += seriesCount;
            counter = 0;
        }
        
        ohlcDataSeries.appendX(SCIGeneric(countUpdater + counter), open: ohlcDataSeries.openValues().valueAt(Int(counter)), high: ohlcDataSeries.highValues().valueAt(Int(counter)), low: ohlcDataSeries.lowValues().valueAt(Int(counter)), close: ohlcDataSeries.closeValues().valueAt(Int(counter)))
        
        mountainDataSeries.appendX(SCIGeneric(countUpdater + counter), y: ohlcDataSeries.highValues().valueAt(Int(counter)))
        
        let priceClose : Double = SCIGenericDouble(ohlcDataSeries.closeValues().valueAt(Int(counter)));
        let priceOpen : Double = SCIGenericDouble(ohlcDataSeries.openValues().valueAt(Int(counter)));
        _lastMarker.position = SCIGeneric(priceClose)
        if (priceClose > priceOpen) {
            _lastMarker.style.backgroundColor = _lastUpColor
        } else {
            _lastMarker.style.backgroundColor = _lastDownColor
        }
//        _averageMarker.position = SCIGeneric(msa.current);
        
        sciChartView1.chartSurface.invalidateElement()
        sciChartView2.chartSurface.invalidateElement()
        
        counter+=1
        
    }
    
    
    // MARK: Overrided Functions
    
    override func awakeFromNib() {
        super.awakeFromNib()
        completeConfiguration()
    }
    
    // MARK: Private Functions
    
    private func configureChartSuraface() {
        
        sciChartView1.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView1)
        
        sciChartView2.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView2)
        
        let layoutDictionary = ["SciChart1" : sciChartView1, "SciChart2" : sciChartView2]
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart1]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart2]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|-(0)-[SciChart1]-(5)-[SciChart2(150)]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
    }
    
    
    private func addAxis() {
        
        let axisStyle = sciChartView1.generateDefaultAxisStyle()
        
        sciChartView1.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: sciChartView1.axisXId), isXAxis: true)
        sciChartView1.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: sciChartView1.axisYId), isXAxis: false)
        
        sciChartView2.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: sciChartView2.axisXId, isVisible: false), isXAxis: true)
        sciChartView2.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: sciChartView2.axisYId, isVisible: false), isXAxis: false)
        
    }
    
    private func getCandleRenderSeries(isReverse: Bool,
                                       upBodyBrush: SCIBrushSolid,
                                       upWickPen: SCIPenLinearGradient,
                                       downBodyBrush: SCIBrushSolid,
                                       downWickPen: SCIPenLinearGradient,
                                       count: Int) -> SCIFastOhlcRenderableSeries {
        
        DataManager.loadPriceData(ohlcDataSeries,
                                  fileName: "FinanceData",
                                  isReversed: isReverse,
                                  count: Int32(count))
        
        ohlcDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let candleRendereSeries = SCIFastOhlcRenderableSeries()
        candleRendereSeries.xAxisId = sciChartView1.axisXId
        candleRendereSeries.yAxisId = sciChartView1.axisYId
        candleRendereSeries.dataSeries = ohlcDataSeries
        candleRendereSeries.style.upWickPen = upWickPen
        candleRendereSeries.style.downWickPen = downWickPen
        
        return candleRendereSeries
    }
    
    private func getMountainRenderSeries(withBrush brush:SCIBrushLinearGradient, and pen: SCIPenSolid) -> SCIFastMountainRenderableSeries {
        
        mountainDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        for i in 0...seriesCount {
            mountainDataSeries.appendX(SCIGeneric(i), y: ohlcDataSeries.highValues().valueAt(Int(i)))
        }
        
        let mountainRenderSeries = SCIFastMountainRenderableSeries()
        mountainRenderSeries.style.areaBrush = brush
        mountainRenderSeries.style.borderPen = pen
        mountainRenderSeries.xAxisId = sciChartView2.axisXId
        mountainRenderSeries.yAxisId = sciChartView2.axisYId
        mountainRenderSeries.dataSeries = mountainDataSeries
        
        return mountainRenderSeries
        
    }
    
    private func addModifiers() {
        sciChartView1.addDefaultModifiers()
        sciChartView2.addDefaultModifiers()
        
        _lastMarker.yAxisId = "yAxis"
        _lastMarker.coordMode = .Absolute;
        _lastMarker.style.backgroundColor = _lastUpColor;
        
        _averageMarker.yAxisId = "yAxis"
        _averageMarker.coordMode = .Absolute;
        _averageMarker.style.backgroundColor = UIColor.fromColorCode(0xFF00a5ff);
        
//        sciChartView1.chartSurface.annotation = SCIAnnotationGroup(childAnnotations:[
//            _averageMarker,
//            _lastMarker
//            ]);
    }
    
    private func addDataSeries() {
        let upBrush = SCIBrushSolid(colorCode: 0xFF0f9cff)
        let downBrush = SCIBrushSolid(colorCode: 0xFF66ffff)
        let upWickPen = SCIPenLinearGradient(colorCodeStart: 0xFF4440a6, finish: 0xFF4440a6, direction: .Vertical, width: 1.0)
        let downWickPen = SCIPenLinearGradient(colorCodeStart: 0xFF3ba13d, finish: 0xFF3ba13d, direction: .Vertical, width: 1.0)
        
        sciChartView1.chartSurface.attachRenderableSeries(getCandleRenderSeries(false, upBodyBrush: upBrush, upWickPen: upWickPen, downBodyBrush: downBrush, downWickPen: downWickPen, count: Int(seriesCount)))
        sciChartView1.chartSurface.invalidateElement()
        
        let brush = SCIBrushLinearGradient(colorCodeStart: 0x888f663a,
                                           finish: 0xff4f3820,
                                           direction: .Vertical)
        let pen = SCIPenSolid(colorCode: 0xff8f663a, width: 0.3)
        
        sciChartView2.chartSurface.attachRenderableSeries(getMountainRenderSeries(withBrush: brush, and: pen))
        sciChartView2.chartSurface.invalidateElement()
        
    }
    
    
    // MARK: Internal Functions
    
    func completeConfiguration() {
        configureChartSuraface()
        addAxis()
        addModifiers()
        addDataSeries()
    }
    
    
}
