//
//  DataManager.m
//  SciChartDemo
//
//  Created by Admin on 09.11.15.
//  Copyright © 2015 SciChart Ltd. All rights reserved.
//

#import "DataManager.h"
#import <SciChart/SciChart.h>

@implementation DataManager

+(void) loadXyData:(id<SCIXyDataSeries>)data From:(double)min To:(double)max Random:(DataManagerRandom)params {
    for (int j = min; j < max; j++) {
        double yValue = randf(j * params.minRandMul + params.minRandAdd, j * params.maxRandMul + params.maxRandAdd);
        yValue += j*params.addMul + params.add;
        if ( ([data xType] == SCIDataType_DateTime)
            && ([data yType] == SCIDataType_DateTime) )
        {
            [data appendX:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:j]) Y:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:yValue])];
        } else if ([data xType] == SCIDataType_DateTime) {
            [data appendX:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:j]) Y:SCIGeneric(yValue)];
        } else if ([data yType] == SCIDataType_DateTime) {
            [data appendX:SCIGeneric(j) Y:SCIGeneric([NSDate dateWithTimeIntervalSinceNow:yValue])];
        } else {
            [data appendX:SCIGeneric(j) Y:SCIGeneric(yValue)];
        }
    }
}

+(void) loadXyData:(id<SCIXyDataSeries>)data From:(double)min To:(double)max Function:(XyDataFunction)func {
    for (int j = min; j < max; j++) {
        func(data,j);
    }
}

+(void) loadOhlcData:(id<SCIOhlcDataSeries>)data From:(double)min To:(double)max Function:(OhlcDataFunction)func {
    for (int j = min; j < max; j++) {
        func(data,j);
    }
}

+(void) loadPriceData:(id<SCIOhlcDataSeries>) dataSeries
             fileName:(NSString*) fileName
           isReversed:(BOOL) reversed
                count:(int) count{
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:fileName
                                                         ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile:filePath
                                               encoding:NSUTF8StringEncoding
                                                  error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    NSArray* subItems;
    if(!reversed)
        for (int i=0; i<count; i++) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            [dataSeries appendX:SCIGeneric(i)
                           Open:SCIGeneric([subItems[1] floatValue])
                           High:SCIGeneric([subItems[2] floatValue])
                            Low:SCIGeneric([subItems[3] floatValue])
                          Close:SCIGeneric([subItems[4] floatValue])];
        }
    else{
        int j=0;
        for (int i=count-1; i>=0; i--) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            [dataSeries appendX:SCIGeneric(j)
                           Open:SCIGeneric([subItems[1] floatValue])
                           High:SCIGeneric([subItems[2] floatValue])
                            Low:SCIGeneric([subItems[3] floatValue])
                          Close:SCIGeneric([subItems[4] floatValue])];
            ++j;
        }
    }
}

+(void) loadDataFromFile:(id<SCIXyDataSeries>) dataSeries
                fileName:(NSString*) fileName
              startIndex:(int) startIndex
               increment:(int) increment
                 reverse:(BOOL)reverse
{
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:fileName
                                                         ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile: filePath
                                               encoding:NSUTF8StringEncoding
                                                  error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    NSArray* subItems;
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"YYYY-MM-dd"];
    
    if (reverse) {
        for (int i = (int)(items.count - 1); i >= startIndex; i -= increment) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            NSDate * date =[dateFormatter dateFromString: subItems[0]];
            
            [dataSeries appendX:SCIGeneric(date)
                              Y: SCIGeneric([subItems[1] doubleValue])];
        }
    } else {
        for (int i = startIndex; i < items.count; i += increment) {
            subItems = [items[i] componentsSeparatedByString:@","];
            
            NSDate * date =[dateFormatter dateFromString: subItems[0]];
            
            [dataSeries appendX:SCIGeneric(date)
                              Y: SCIGeneric([subItems[1] doubleValue])];
        }
    }
}

+(void) loadDataFromFile:(id<SCIXyDataSeries>) dataSeries
                fileName:(NSString*) fileName{
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:fileName ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile: filePath encoding:NSUTF8StringEncoding error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    
      for (int i=0; i<items.count; i++) {
        [dataSeries appendX:SCIGeneric(i) Y: SCIGeneric([items[i] floatValue])];
    }
}

+ (void)putDefaultDataMultiPaneIntoDataSeries:(id<SCIXyDataSeries>)dataSeries dataCount:(NSInteger)dataCount {
    
    SCIGenericType yData;
    yData.floatData = arc4random_uniform(100);
    yData.type = SCIDataType_Float;
    int32_t prevoius = 1;
    for (int i = 0; i < dataCount; i++) {
        prevoius = randi(prevoius, prevoius+10);
        NSDate *date = [NSDate dateWithTimeIntervalSinceNow:60*60*24*prevoius];
        SCIGenericType xData = SCIGeneric(date);
        float value = yData.floatData + randf(-5.0, 5.0);
        yData.floatData = value;
        [dataSeries appendX:xData Y:yData];
    }
    
}

+ (NSArray<SCDMultiPaneItem *> *)loadPaneStockData {
    
    NSInteger count = 3000;
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:@"EURUSD_Daily"
                                                         ofType:@"txt"];
    
    NSString *data = [NSString stringWithContentsOfFile:filePath
                                               encoding:NSUTF8StringEncoding
                                                  error:nil];
    
    NSArray* items = [data componentsSeparatedByString:@"\n"];
    NSArray* subItems;
    
    NSMutableArray *array = [[NSMutableArray alloc] initWithCapacity:count];
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"YYYY-MM-dd"];
    
    for (int i = 0; i < count; i++) {
        subItems = [items[i] componentsSeparatedByString:@","];
        
        SCDMultiPaneItem *item = [SCDMultiPaneItem new];
        item.dateTime = [dateFormatter dateFromString: subItems[0]];
        item.open = [subItems[1] doubleValue];
        item.high = [subItems[2] doubleValue];
        item.low = [subItems[3] doubleValue];
        item.close = [subItems[4] doubleValue];
        item.volume = [subItems[5] doubleValue];
        
        [array addObject:item];
        
    }
    
    return array;
}

@end

@implementation SCDMultiPaneItem

@end

@implementation SCDMovingAverage {
    int _lenght;
    int _circIndex;
    bool _filled;
    double _current;
    double _oneOverLength;
    NSMutableArray <NSNumber *> *_circularBuffer;
    double _total;
}

- (instancetype)initWithLength:(int)length {
    self = [super init];
    if (self) {
        _lenght = length;
        _oneOverLength = 1.0/(double)length;
        _circularBuffer = [[NSMutableArray alloc] initWithCapacity:_lenght];
        _circIndex = -1;
        _current = NAN;
    }
    return self;
}

- (void)update:(double)value {
    
    double lostValue = _circIndex < _circularBuffer.count ? _circularBuffer[_circIndex].doubleValue : 0.0;
    _circularBuffer[_circIndex] = @(value);
    
    // Maintain totals for Push function
    _total += value;
    _total -= lostValue;
    
    // If not yet filled, just return. Current value should be double.NaN
    if (!_filled)
    {
        _current = NAN;
        return;
    }
    
    // Compute the average
    double average = 0.0;
    for (int i = 0; i < _circularBuffer.count; i++)
    {
        average += _circularBuffer[i].doubleValue;
    }
    
    _current = average * _oneOverLength;
    
}

- (SCDMovingAverage*)push:(double)value {
    
    if (++_circIndex == _lenght) {
        _circIndex = 0;
    }
    
    
    double lostValue = _circIndex < _circularBuffer.count ? _circularBuffer[_circIndex].doubleValue : 0.0;
    _circularBuffer[_circIndex] = @(value);
    
    _total += value;
    _total -= lostValue;
    
    if (!_filled && _circIndex != _lenght-1) {
        _current = NAN;
        return self;
    }
    else {
        _filled = true;
    }
    
    _current = _total * _oneOverLength;
    
    return self;
    
}

@end

@implementation SCDMcadPointItem

@end
